package com.ouling.weibo;

import java.text.SimpleDateFormat;
import java.util.List;

import android.app.NotificationManager;
import android.content.Context;
import android.content.Intent;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.BaseAdapter;
import android.widget.ImageView;
import android.widget.TextView;
import com.ouling.*;
import com.ouling.weibo.OAuth.OAuthConstant;
import com.ouling.weibo.basic.Comment;
import com.ouling.weibo.listener.OnGetMoreListener;
import com.ouling.weibo.listener.OnRefreshListener;
import com.ouling.weibo.basic.Paging;
import com.ouling.weibo.basic.Status;
import com.ouling.weibo.basic.Weibo;
import com.ouling.weibo.basic.WeiboException;
import com.ouling.weibo.basic.WeiboResponse;
import com.ouling.weibo.util.AsyncImageView;
import com.ouling.weibo.util.AutoGetMoreListView;
import com.ouling.weibo.util.HighLightTextView;
import com.ouling.weibo.util.ImageRel;

/**
 * ʾ/@/΢б
 * 
 * 
 */
public class WeiboAdapter extends BaseAdapter implements OnRefreshListener,
		OnGetMoreListener {

	public static final SimpleDateFormat sdf = new SimpleDateFormat(
			"yyyy-MM-dd HH:mm");

	private List<WeiboResponse> curList;
	private Context context;
	private AutoGetMoreListView autoGetMoreListView;
	private int type; // 0 for յ
						// 1 for 
						// 2 for @
						// 3 for ΢
	private String cid; // ǰ鿴ָ˵΢

	static class ViewHolder {

		AsyncImageView wbicon;
		HighLightTextView wbtext;
		TextView wbtime;
		TextView wbuser;
		ImageView wbimage;
	}

	public WeiboAdapter(List<WeiboResponse> curList, Context context, int type,
			AutoGetMoreListView autoGetMoreListView) {
		this.curList = curList;
		this.context = context;
		this.type = type;
		this.autoGetMoreListView = autoGetMoreListView;
		this.autoGetMoreListView.setOnRefreshListener(this);
		this.autoGetMoreListView.setOnGetMoreListener(this);
		this.cid = null;

		System.setProperty("weibo4j.oauth.consumerKey", Weibo.CONSUMER_KEY);
		System.setProperty("weibo4j.oauth.consumerSecret",
				Weibo.CONSUMER_SECRET);
	}

	@Override
	public int getCount() {
		return curList.size();
	}

	@Override
	public WeiboResponse getItem(int position) {
		return curList.get(position);
	}

	@Override
	public long getItemId(int position) {
		return position;
	}

	@Override
	public View getView(int position, View convertView, ViewGroup parent) {
		ViewHolder holder;
		if (convertView == null) {
			convertView = LayoutInflater.from(context).inflate(
					R.layout.timeline, null);
			AsyncImageView wbicon = (AsyncImageView) convertView
					.findViewById(R.id.wbicon);
			HighLightTextView wbtext = (HighLightTextView) convertView
					.findViewById(R.id.wbtext);
			TextView wbtime = (TextView) convertView.findViewById(R.id.wbtime);
			TextView wbuser = (TextView) convertView.findViewById(R.id.wbuser);
			ImageView wbimage = (ImageView) convertView
					.findViewById(R.id.wbimage);

			holder = new ViewHolder();
			holder.wbicon = wbicon;
			holder.wbimage = wbimage;
			holder.wbtext = wbtext;
			holder.wbtime = wbtime;
			holder.wbuser = wbuser;
			convertView.setTag(holder);
		} else {
			holder = (ViewHolder) convertView.getTag();
		}

		holder.wbimage.setVisibility(View.GONE);
		holder.wbicon.setProgressBitmaps(ImageRel.getBitmaps_avatar(context));

		String screenName = "";
		String userID = "";
		switch (type) {
			case 0:
			case 1:
				Comment comment = (Comment) curList.get(position);
				userID = comment.getUser().getId() + "";
				String commentText = comment.getText();
				if (comment.getReply_comment() != null) {
					commentText = commentText
							+ "\n\nظ@"
							+ comment.getReply_comment().getUser()
									.getScreenName() + " : "
							+ comment.getReply_comment().getText();

					if (comment.getStatus() != null)
						commentText = commentText + "\n\nԭ΢: @"
								+ comment.getStatus().getUser().getScreenName()
								+ ": " + comment.getStatus().getText();

					if (comment.getStatus().getRetweeted_status() != null)
						commentText = commentText
								+ "\nת: @"
								+ comment.getStatus().getRetweeted_status()
										.getUser().getScreenName()
								+ ": "
								+ comment.getStatus().getRetweeted_status()
										.getText();

				} else if (comment.getStatus() != null) {
					commentText = commentText + "\n\n@"
							+ comment.getStatus().getUser().getScreenName()
							+ " ΢: " + comment.getStatus().getText();

					if (comment.getStatus().getRetweeted_status() != null)
						commentText = commentText
								+ "\nת: @"
								+ comment.getStatus().getRetweeted_status()
										.getUser().getScreenName()
								+ ": "
								+ comment.getStatus().getRetweeted_status()
										.getText();
				}

				if (comment.getStatus() != null) {
					if (comment.getStatus().getRetweeted_status() != null) {
						if (!TextUtils.isEmpty(comment.getStatus()
								.getRetweeted_status().getThumbnail_pic()))
							holder.wbimage.setVisibility(View.VISIBLE);
					} else if (!TextUtils.isEmpty(comment.getStatus()
							.getThumbnail_pic())) {
						holder.wbimage.setVisibility(View.VISIBLE);
					}
				}

				holder.wbtext.setText(commentText);
				holder.wbicon.setUrl(comment.getUser().getProfileImageURL()
						.toString());
				holder.wbtime.setText(sdf.format(comment.getCreatedAt()));
				screenName = comment.getUser().getScreenName();
				break;

			case 2:
			case 3:
				Status status = (Status) curList.get(position);
				userID = status.getUser().getId() + "";
				holder.wbicon.setUrl(status.getUser().getProfileImageURL()
						.toString());

				String text = status.getText();
				if (status.getRetweeted_status() != null)
					text = text
							+ "\n\nת: @"
							+ status.getRetweeted_status().getUser()
									.getScreenName() + " :"
							+ status.getRetweeted_status().getText();
				holder.wbtext.setText(text);
				holder.wbtime.setText(sdf.format(status.getCreatedAt()));
				screenName = status.getUser().getScreenName();

				if (status.getRetweeted_status() != null) {
					if (!TextUtils.isEmpty(status.getRetweeted_status()
							.getThumbnail_pic()))
						holder.wbimage.setVisibility(View.VISIBLE);
				} else if (!TextUtils.isEmpty(status.getThumbnail_pic()))
					holder.wbimage.setVisibility(View.VISIBLE);
				break;
		}

		final String cid = userID;
		holder.wbicon.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				Intent intent = new Intent(context, UserInfo.class);
				intent.putExtra("cid", cid);
				context.startActivity(intent);
			}
		});

		if (screenName.length() > 6) {
			screenName = screenName.substring(0, 6) + "...";
		}
		holder.wbuser.setText(screenName);
		return convertView;
	}

	public void setCurList(List<WeiboResponse> curList) {
		this.curList = curList;
	}

	public void setType(int type) {
		this.type = type;
	}

	@Override
	public void onGetMore() {
		new Thread(new Runnable() {

			@Override
			public void run() {
				Paging paging = new Paging(2);
				if (curList.size() > 0) {
					Weibo weibo = OAuthConstant.getInstance().getWeibo();
					try {
						switch (type) {
							case 0:
								paging.setMaxId(((Comment) curList.get(curList
										.size() - 1)).getId());
								curList.addAll(weibo.getCommentsToMe(paging));
								break;

							case 1:
								paging.setMaxId(((Comment) curList.get(curList
										.size() - 1)).getId());
								curList.addAll(weibo.getCommentsByMe(paging));
								break;

							case 2:
								paging.setMaxId(((Status) curList.get(curList
										.size() - 1)).getId());
								curList.addAll(weibo.getMentions(paging));
								break;

							case 3:
								paging.setMaxId(((Status) curList.get(curList
										.size() - 1)).getId());
								if (TextUtils.isEmpty(cid))
									curList.addAll(weibo
											.getFriendsTimeline(paging));
								else
									curList.addAll(weibo.getUserTimeline(cid,
											paging));
								break;
						}
					} catch (WeiboException e) {
						e.printStackTrace();
					}
				}
				autoGetMoreListView.getMoreFinished();
			}
		}).start();
	}

	@Override
	public void onRefresh() {
		new Thread(new Runnable() {

			@Override
			public void run() {
				Weibo weibo = OAuthConstant.getInstance().getWeibo();
				NotificationManager notificationManager = (NotificationManager) context
						.getSystemService(Context.NOTIFICATION_SERVICE);
				try {
					switch (type) {
						case 0:
							List<Comment> newCommentToMe = weibo
									.getCommentsToMe();
							curList.clear();
							curList.addAll(newCommentToMe);
							notificationManager
									.cancel(MainActivity.UNREAD_COMMENT);
							weibo.resetCount(1);
							break;

						case 1:
							List<Comment> newCommentByMe = weibo
									.getCommentsByMe();
							curList.clear();
							curList.addAll(newCommentByMe);
							break;

						case 2:
							List<Status> newMentions = weibo.getMentions();
							curList.clear();
							curList.addAll(newMentions);

							notificationManager
									.cancel(MainActivity.UNREAD_MENTION);
							weibo.resetCount(2);
							break;

						case 3:
							List<Status> newTimeline;
							if (TextUtils.isEmpty(cid))
								newTimeline = weibo.getFriendsTimeline();
							else
								newTimeline = weibo.getUserTimeline(cid);
							curList.clear();
							curList.addAll(newTimeline);
							break;
					}
				} catch (WeiboException e) {
					e.printStackTrace();
				}
				autoGetMoreListView.refreshFinished();
			}
		}).start();
	}

	/**
	 * @param cid
	 *            the cid to set
	 */
	public void setCid(String cid) {
		this.cid = cid;
	}
}